(function(window, undefined) {
    v11n = {
        version: "0.0.1"
    };
    if (!Object.keys) {
        Object.keys = function() {
            var hasOwnProperty = Object.prototype.hasOwnProperty, hasDontEnumBug = !{
                toString: null
            }.propertyIsEnumerable("toString"), dontEnums = [ "toString", "toLocaleString", "valueOf", "hasOwnProperty", "isPrototypeOf", "propertyIsEnumerable", "constructor" ];
            return function(obj) {
                if (typeof obj !== "object" && typeof obj !== "function" || obj === null) throw new TypeError("Object.keys called on non-object");
                var result = [];
                for (var prop in obj) {
                    if (hasOwnProperty.call(obj, prop)) result.push(prop);
                }
                if (hasDontEnumBug) {
                    for (var i = 0; i < dontEnums.length; i++) {
                        if (hasOwnProperty.call(obj, dontEnums[i])) result.push(dontEnums[i]);
                    }
                }
                return result;
            };
        }();
    }
    if (!Array.range) {
        Array.range = function(start, stop, step) {
            if (arguments.length <= 1) {
                stop = start || 0;
                start = 0;
            }
            step = arguments[2] || 1;
            var range = new Array(Math.max(Math.ceil(Math.abs(stop - start) / step), 0));
            for (var i = 0; i < range.length; i++) {
                range[i] = start;
                start += step;
            }
            return range;
        };
    }
    v11n.layout = {};
    v11n.layout.depth = function() {
        var width = 500, height = 500, margin = 30, min = 0, max = 10, interpolate = "linear", dispatch = null;
        function depth(selection) {
            selection.each(function(d) {
                d = mapper(d);
                var ys = yScale(d.values);
                var y = d3.scale.linear().range([ margin, height - d.values.length * margin ]).domain(ys);
                var range = Array.range(+ys[0], +ys[1]);
                var graph = d3.select(this).selectAll("svg").data([ 0 ]);
                graph.enter().append("svg").attr("width", "0").attr("height", "0").attr("class", "v11n-depth-chart").transition().attr("width", width).attr("height", height);
                var yAxis = graph.selectAll(".v11n-y-axis").data([ d.values ]);
                yAxis.enter().append("g").attr("class", "v11n-y-axis").attr("transform", "translate(" + margin + ", 0)");
                yAxis.exit().remove();
                yAxis.transition().call(d3.svg.axis().scale(y).orient("left"));
                var depthBins = graph.selectAll(".v11n-depth-bin").data(range);
                depthBins.enter().append("rect").attr("class", "v11n-depth-bin").style("color", "white").style("opacity", "0").style("cursor", "pointer").on("mouseover.v11n", function(d, i) {
                    d3.select(this).transition().style("color", "grey").style("opacity", "0.6");
                }).on("mouseout.v11n", function(d, i) {
                    d3.select(this).transition().style("color", "white").style("opacity", "0");
                }).on("click.v11n", function(d, i) {
                    d3.selectAll(".v11n-selected-bin").classed("v11n-selected-bin", false);
                    d3.select(this).classed("v11n-selected-bin", true);
                    dispatch.changeDepth.call(this, i);
                });
                depthBins.exit().remove();
                depthBins.transition().attr("x", margin).attr("y", function(d) {
                    return y(d);
                }).attr("width", width - 2 * margin).attr("height", (height - margin * (d.values.length + 1)) / range.length);
                var chart = graph.selectAll(".v11n-chart").data(d.values, function(d) {
                    return d.key;
                });
                chart.enter().append("g").attr("class", function(d) {
                    return "v11n-chart " + d.key;
                });
                chart.exit().transition().style("opacity", 0).remove();
                var xAxis = chart.selectAll(".v11n-x-axis").data(function(d) {
                    return [ d ];
                }, function(d) {
                    return d.key;
                });
                xAxis.enter().append("g").attr("class", "v11n-x-axis").attr("transform", function(d) {
                    return "translate(0, " + (height + 5) + ")";
                });
                xAxis.transition().call(function(selection) {
                    selection.each(function(d) {
                        d3.select(this).call(d3.svg.axis().scale(d.xScale).orient("bottom"));
                    });
                }).attr("transform", function(d, i, j) {
                    return "translate(0, " + (height - margin * (j + 1)) + ")";
                });
                var line = chart.selectAll(".v11n-line").data(function(d) {
                    return [ d ];
                }, function(d) {
                    return d.key;
                });
                line.enter().append("path").attr("class", "v11n-line");
                line.transition().attr("d", function(d) {
                    return d3line(d.xScale, y)(d.values);
                });
                var area = chart.selectAll(".v11n-area").data(function(d) {
                    return [ d ];
                }, function(d) {
                    return d.key;
                });
                area.enter().append("path").attr("class", "v11n-area");
                area.transition().attr("d", function(d) {
                    return d3area(d.xScale, y)(d.values);
                });
            });
        }
        function d3line(x, y) {
            return d3.svg.line().x(function(d) {
                return x(d.x);
            }).y(function(d) {
                return y(d.y);
            }).interpolate(interpolate);
        }
        function d3area(x, y) {
            return d3.svg.area().x0(function(d) {
                return x(d.x0);
            }).x1(function(d) {
                return x(d.x1);
            }).y(function(d) {
                return y(d.y);
            }).interpolate(interpolate);
        }
        function yScale(values) {
            var yValues = values.reduce(function(a, b) {
                return a.concat(b.values.reduce(function(a, b) {
                    return a.concat([ b.y ]);
                }, []));
            }, []);
            return d3.extent(yValues);
        }
        function xScale(values) {
            var xValues = values.reduce(function(a, b) {
                return a.concat([ b.x0, b.x, b.x1 ]);
            }, []);
            return d3.scale.linear().range([ margin, width - margin ]).domain([ Math.min(d3.min(xValues), min), Math.max(d3.max(xValues), max) ]);
        }
        function depthBin(values) {
            var depthSums = {}, minBoundSum = {}, maxBoundSum = {};
            var depthCounts = {};
            values.forEach(function(d) {
                var val = Math.floor(d.y);
                if (!depthCounts[val]) {
                    depthCounts[val] = depthSums[val] = minBoundSum[val] = maxBoundSum[val] = 0;
                }
                depthCounts[val]++;
                depthSums[val] += d.x;
                minBoundSum[val] += d.x0;
                maxBoundSum[val] += d.x1;
            });
            return Object.keys(depthCounts).map(function(d) {
                return {
                    x: depthSums[d] / depthCounts[d],
                    y: +d,
                    x0: minBoundSum[d] / depthCounts[d],
                    x1: maxBoundSum[d] / depthCounts[d]
                };
            });
        }
        function mapper(data) {
            data.values.map(function(d) {
                d.values = depthBin(d.values);
                d.xScale = xScale(d.values);
                return d;
            });
            return data;
        }
        depth.remove = function(selection) {
            selection.each(function(d) {
                d3.select(this).selectAll("svg").transition().attr("height", 0).attr("width", 0).remove();
            });
        };
        depth.width = function(value) {
            if (!arguments.length) return width;
            width = value;
            return depth;
        };
        depth.height = function(value) {
            if (!arguments.length) return height;
            height = value;
            return depth;
        };
        depth.margin = function(value) {
            if (!arguments.length) return margin;
            margin = value;
            return depth;
        };
        depth.min = function(value) {
            if (!arguments.length) return min;
            min = value;
            return depth;
        };
        depth.max = function(value) {
            if (!arguments.length) return max;
            max = value;
            return depth;
        };
        depth.interpolate = function(value) {
            if (!arguments.length) return interpolate;
            interpolate = value;
            return depth;
        };
        depth.dispatch = function(value) {
            if (!arguments.length) return dispatch;
            dispatch = value;
            return depth;
        };
        return depth;
    };
    v11n.layout.boxwhisker = function() {
        var width = 160, height = 500, vMargin = 30, hMargin = 30, mapping = function(d) {
            return d.y;
        };
        function boxwhisker(selection) {
            selection.each(function(d) {
                d = d.values[0];
                d = d.values.map(mapping).sort(d3.ascending);
                var x = d3.scale.quantize().domain([ 0, 100 ]).range([ 0, hMargin, width / 2, width - hMargin, width ]), y = d3.scale.linear().domain([ Math.min(0, d[0]), d[d.length - 1] ]).range([ height - vMargin, vMargin ]);
                var median = d3.quantile(d, .5), quartileRange = [ d3.quantile(d, .25), d3.quantile(d, .75) ], IQR = 1.5 * (quartileRange[1] - quartileRange[0]), fn = function(d) {
                    return d < quartileRange[0] - IQR || d > quartileRange[1] + IQR;
                }, outliers = d.filter(function(d) {
                    return fn(d);
                }), mean = d3.mean(d), extent = d3.extent(d.filter(function(d) {
                    return !fn(d);
                }));
                var svg = d3.select(this).selectAll(".v11n-bw").data([ 0 ]);
                svg.enter().append("svg").attr("xmlns", "http://www.w3.org/2000/svg").attr("version", "1.1").attr("preserveAspectRatio", "none").attr("class", "v11n-bw");
                svg.attr("viewBox", "0 0 " + width + " " + height);
                var line = svg.selectAll(".v11n-quartile-range").data([ x(25), x(75) ]);
                line.enter().append("line").attr("class", "v11n-quartile-range");
                line.transition().attr("x1", function(d) {
                    return d;
                }).attr("x2", function(d) {
                    return d;
                }).attr("y1", y(quartileRange[0])).attr("y2", y(quartileRange[1]));
                var circle = svg.selectAll(".v11n-mean").data([ mean ]);
                circle.enter().append("circle").attr("class", "v11n-mean");
                circle.transition().attr("cy", function(d) {
                    return y(d);
                }).attr("r", (x(75) - x(25)) / 16).attr("cx", x(50));
                var connection = svg.selectAll(".v11n-connection").data(extent);
                connection.enter().append("line").attr("class", "v11n-connection");
                connection.transition().attr("x1", x(50)).attr("y1", function(d) {
                    return y(d);
                }).attr("x2", x(50)).attr("y2", function(d, i) {
                    return y(quartileRange[i]);
                });
                var outlier = svg.selectAll(".v11n-outlier").data(outliers);
                outlier.enter().append("circle").attr("class", "v11n-outlier").attr("cy", function(d) {
                    return d < extent[0] ? height + 20 : -10;
                });
                outlier.exit().transition().style("opactiy", "0").attr("cy", function(d) {
                    return d3.select(this).attr("cy") >= height / 2 ? height + 20 : -10;
                }).remove();
                outlier.transition().attr("cy", function(d) {
                    return y(d);
                }).attr("r", (x(75) - x(25)) / 32).attr("cx", x(50));
                extent.splice(1, 0, quartileRange[0], median, quartileRange[1]);
                var feature = svg.selectAll(".v11n-feature").data(extent), featureEnter = feature.enter().append("g").attr("class", "v11n-feature");
                featureEnter.append("line");
                featureEnter.append("text");
                feature.select("line").transition().attr("x1", x(25)).attr("x2", x(75)).attr("y1", function(d) {
                    return y(d);
                }).attr("y2", function(d) {
                    return y(d);
                });
                feature.select("text").transition().attr("dy", ".32em").attr("x", function(d, i) {
                    return i % 2 ? x(25) : x(75);
                }).attr("y", function(d) {
                    return y(d);
                }).text(function(d) {
                    return Math.round(d * 100) / 100;
                }).style("text-anchor", function(d, i) {
                    return i % 2 ? "end" : "start";
                });
            });
        }
        boxwhisker.width = function(value) {
            if (!arguments.length) return width;
            width = value;
            return boxwhisker;
        };
        boxwhisker.height = function(value) {
            if (!arguments.length) return height;
            height = value;
            return boxwhisker;
        };
        boxwhisker.margin = function(vertical, horizontal) {
            if (!arguments.length) return [ vMargin, hMargin ];
            vMargin = vertical;
            if (horizontal === undefined) hMargin = vertical; else hMargin = horizontal;
            return boxwhisker;
        };
        boxwhisker.mapping = function(value) {
            if (!arguments.length) return mapping;
            mapping = value;
            return boxwhisker;
        };
        return boxwhisker;
    };
    v11n.layout.sparkline = function() {
        var height = 30, width = 800, max = 100, min = 0, margin = 2, dateFormat = d3.time.format("%x");
        xScale = null, yScale = null, interpolate = "linear";
        function sparkline(selection) {
            selection.each(function(d) {
                d = d.values;
                var x = xScale ? xScale : d3.time.scale().domain(d.map(function(d) {
                    return d.x;
                })), y = yScale ? yScale : d3.scale.linear().domain(d3.extent(d, function(d) {
                    return d.y;
                }));
                x.range([ margin, width - margin ]);
                y.range([ height - margin, margin ]);
                var line = d3.svg.line().x(function(d) {
                    return x(d.x);
                }).y(function(d) {
                    return y(d.y);
                }).interpolate(interpolate);
                var container = d3.select(this);
                var svg = container.selectAll("svg").data([ 0 ]);
                svg.enter().append("svg").attr("width", width).attr("height", height);
                var g = svg.selectAll("g").data([ 0 ]);
                g.enter().append("g");
                var path = g.selectAll(".v11n-line").data([ d ]);
                path.enter().append("path").attr("class", "v11n-line").attr("fill", "none").attr("stroke-width", "1").attr("stroke", "#08C");
                path.transition().attr("d", function(d) {
                    return line(d);
                });
                var dataPoints = g.selectAll(".v11n-dataPoint").data(d);
                dataPoints.enter().append("circle").attr("class", "v11n-dataPoint").attr("r", 5).style("opacity", 0);
                dataPoints.attr("cx", function(d) {
                    return x(d.x);
                }).attr("cy", function(d) {
                    return y(d.y);
                });
                var clearText = g.selectAll(".v11n-clearText").data([ 0 ]);
                clearText.enter().append("text").attr("class", "v11n-clearText").attr("x", margin).attr("y", margin + 12);
                dataPoints.on("mouseover", function(d) {
                    d3.select(this).style("opacity", 1);
                    g.selectAll(".v11n-clearText").text(dateFormat(d.x) + ", " + d.y);
                }).on("mouseout", function(d) {
                    d3.select(this).style("opacity", 0);
                    g.selectAll(".v11n-clearText").text("");
                });
                sparkline.exts.forEach(function(d) {
                    d.call(sparkline, svg);
                });
            });
        }
        sparkline.height = function(value) {
            if (!arguments.length) return height;
            height = value;
            return sparkline;
        };
        sparkline.width = function(value) {
            if (!arguments.length) return width;
            width = value;
            return sparkline;
        };
        sparkline.interpolate = function(value) {
            if (!arguments.length) return interpolate;
            interpolate = value;
            return sparkline;
        };
        sparkline.margin = function(value) {
            if (!arguments.length) return margin;
            margin = value;
            return sparkline;
        };
        sparkline.max = function(value) {
            if (!arguments.length) return max;
            max = value;
            return sparkline;
        };
        sparkline.min = function(value) {
            if (!arguments.length) return min;
            min = value;
            return sparkline;
        };
        sparkline.x = function(value) {
            if (!arguments.length) return xScale;
            xScale = value;
            return sparkline;
        };
        sparkline.y = function(value) {
            if (!arguments.length) return yScale;
            yScale = value;
            return sparkline;
        };
        sparkline.ext = function(ext) {
            if (!sparkline.exts) sparkline.exts = [];
            sparkline.exts.push(ext);
        };
        if (v11n.layout.sparkline.extensions) {
            v11n.layout.sparkline.extensions.forEach(function(d) {
                d.call(sparkline);
            });
        }
        return sparkline;
    };
    v11n.layout.sparkline.extend = function(extension) {
        if (!v11n.layout.sparkline.extensions) v11n.layout.sparkline.extensions = [];
        v11n.layout.sparkline.extensions.push(extension);
    };
    v11n.layout.timeSeries = function() {
        var width = 100, height = 100, margin = {
            top: 0,
            right: 0,
            bottom: 0,
            left: 0
        }, padding = {
            top: 30,
            right: 30,
            bottom: 30,
            left: 60
        }, x = null, y = null, interpolate = "linear", min = 0, max = 10, xFormat = function(x) {
            return x;
        }, yFormat = function(y) {
            return y;
        }, sharedXAxis = true, sharedYAxis = true;
        function timeSeries(selection) {
            selection.each(function(data) {
                data.values = data.values.map(function(e) {
                    e.values = e.values.map(function(f) {
                        f.x = new Date(f.x);
                        f.y = +f.y;
                        return f;
                    }).sort(function(a, b) {
                        return a.x - b.x;
                    });
                    return e;
                });
                x = d3.time.scale().range([ padding.left, width - padding.right ]);
                y = d3.scale.linear().range([ height - padding.bottom, padding.top ]);
                var viz = d3.select(this);
                var svg = viz.selectAll("svg").data([ data ], function(d) {
                    return d.key;
                });
                svg.enter().append("svg").attr("xmlns", "http://www.w3.org/2000/svg").attr("version", "1.1").attr("preserveAspectRatio", "none").attr("class", function(e) {
                    return "v11n-time-series " + e.key;
                });
                svg.attr("viewBox", "0 0 " + width + " " + height);
                var arr = data.key.split(" ");
                var plotG = svg.selectAll(".v11n-plotG").data(function(e) {
                    return e.values;
                }, function(k) { return k.key; });
                plotG.enter().append("g").attr("class", function(f) {
                    return "v11n-plotG " + f.key;
                });
                plotG.exit().remove();
                plotG.transition().call(plots, arr[arr.length - 1]);
            });
        }
        function plots(selection, units) {
            selection.each(function(data, i) {
                var bisectDate = d3.bisector(function(d) {
                    return d.x;
                }).left;
                var valueFormat = d3.format(".2r");
                var plot = d3.select(this);
                var x = d3.time.scale().domain(d3.extent(data.values.map(function(d) {
                    return d.x;
                }))).range([ padding.left, width - padding.right ]);
                var y = d3.scale.linear().domain(d3.extent(data.values.map(function(d) {
                    return d.y;
                }).concat([ min ]))).range([ height - padding.bottom, padding.top ]);
                var xAxisG = plot.selectAll(".v11n-x-axis").data([ data.values ]);
                xAxisG.enter().append("g").attr("class", "v11n-x-axis");
                xAxisG.transition().attr("transform", function(d) {
                    return "translate(0, " + (height - padding.bottom) + ")";
                }).call(d3.svg.axis().scale(x).orient("bottom"));
                var yAxisG = plot.selectAll(".v11n-y-axis").data([ data.values ]);
                yAxisG.enter().append("g").attr("class", "v11n-y-axis").attr("transform", "translate(" + padding.left + ", 0)");
                yAxisG.transition().call(d3.svg.axis().scale(y).orient("left").tickFormat(valueFormat));
                var yAxisLabel = plot.selectAll(".v11n-y-axis-label").data([ data ]);
                yAxisLabel.enter().append("text").style("text-anchor", "end").attr("transform", "translate(20, 20), rotate(-90)").text(function(d) {
                    return data.key;
                });
                yAxisLabel.exit().remove();
                var linePath = plot.selectAll(".v11n-line").data([ data.values ]);
                linePath.enter().append("path").attr("class", "v11n-line");
                linePath.transition().attr("d", function(g) {
                    return d3.svg.line().x(function(d) {
                        return x(d.x);
                    }).y(function(d) {
                        return y(d.y);
                    }).interpolate(interpolate)(g);
                });
                var rect = plot.selectAll(".v11n-listener-rect").data([ data.values ]);
                rect.enter().append("rect").attr("class", "v11n-listener-rect").attr("transform", "translate(" + padding.left + ", " + padding.top + ")").attr("width", width - padding.left - padding.right).attr("height", height - padding.top - padding.bottom).style("opacity", 0);
                rect.on("mouseout", function() {
                    focusG.style("display", "none");
                });
                rect.on("mouseover", function() {
                    focusG.style("display", null);
                });
                rect.on("mousemove", mousemove);
                var focusG = plot.selectAll(".v11n-focus").data([ data.values ]);
                var focusGEnter = focusG.enter().append("g").attr("class", "v11n-focus").style("display", "none");
                focusGEnter.append("circle").attr("r", "4.5");
                focusGEnter.append("text").attr("x", 9).attr("dy", ".35em");
                function mousemove() {
                    var x0 = x.invert(d3.mouse(this)[0] + padding.left), i = bisectDate(data.values, x0, 1), d0 = data.values[i - 1], d1 = data.values[i], d = x0 - d0.x > d1.x - x0 ? d1 : d0;
                    focusG.attr("transform", "translate(" + x(d.x) + "," + y(d.y) + ")");
                    focusG.select("text").text(valueFormat(d.y));
                }
            });
        }
        timeSeries.width = function(value) {
            if (!arguments.length) return width;
            width = value;
            return timeSeries;
        };
        timeSeries.height = function(value) {
            if (!arguments.length) return height;
            height = value;
            return timeSeries;
        };
        timeSeries.margin = function(value) {
            if (!arguments.length) return margin;
            applyPadding(value.split(" "), margin);
            return timeSeries;
        };
        timeSeries.padding = function(value) {
            if (!arguments.length) return padding;
            applyPadding(value.split(" "), padding);
            return timeSeries;
        };
        timeSeries.min = function(value) {
            if (!arguments.length) return min;
            min = value;
            return timeSeries;
        };
        timeSeries.max = function(value) {
            if (!arguments.length) return max;
            max = value;
            return timeSeries;
        };
        timeSeries.x = function(value) {
            if (!arguments.length) return x;
            x = value;
            return timeSeries;
        };
        timeSeries.xFormat = function(value) {
            if (!arguments.length) return xFormat;
            xFormat = value;
            return timeSeries;
        };
        timeSeries.y = function(value) {
            if (!arguments.length) return y;
            y = value;
            return timeSeries;
        };
        timeSeries.yFormat = function(value) {
            if (!arguments.length) return yFormat;
            yFormat = value;
            return timeSeries;
        };
        timeSeries.sharedXAxis = function(value) {
            if (!arguments.length) return sharedXAxis;
            sharedXAxis = value;
            return timeSeries;
        };
        timeSeries.sharedYAxis = function(value) {
            if (!arguments.length) return sharedYAxis;
            sharedYAxis = value;
            return timeSeries;
        };
        timeSeries.interpolate = function(value) {
            if (!arguments.length) return interpolate;
            interpolate = value;
            return timeSeries;
        };
        function applyPadding(values, obj) {
            if (values[0]) {
                obj.top = obj.right = obj.bottom = obj.left = +values[0];
            }
            if (values[1]) {
                obj.right = obj.left = +values[1];
            }
            if (values[2]) {
                obj.bottom = +values[2];
            }
            if (values[3]) {
                obj.left = +values[3];
            }
        }
        return timeSeries;
    };
    v11n.util = {};
    v11n.util.addMethod = function(object, name, fn) {
        var old = object[name];
        object[name] = function() {
            if (fn.length = arguments.length) return fn.apply(this, arguments); else if (typeof old == "function") return old.apply(this, arguments);
        };
    };
    v11n.util.call = function(selection, callback) {
        selection.each(function(d) {
            callback(d3.select(this), d);
        });
    };
    Function.prototype.memoized = function() {
        this._values = this._values || {};
        return this._values[JSON.stringify(arguments)] !== undefined ? this._values[JSON.stringify(arguments)] : this._values[JSON.stringify(arguments)] = this.apply(this, arguments);
    };
    Function.prototype.memoize = function() {
        var fn = this;
        return function() {
            return fn.memoized.apply(fn, arguments);
        };
    };
})(window);